/**
 * 
 */
package sim.graphics;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import sim.exception.SRuntimeException;
import sim.loader.SLoaderException;
import sim.loader.texture.STextureLoader;
import sim.util.SLog;
import sim.util.SStringUtil;

/**
 * La classe <b>STextureReader</b> reprsente un lecteur de texture.
 * 
 * @author Simon Vzina
 * @since 2015-11-07
 * @version 2020-06-16
 */
public class STextureReader implements SReader {

  private static final STexture DEFAULT_TEXTURE = null;    
  
  //Carte des textures dj lues (pour ne pas refaire de chargement)
  //Cl de recherche : Nom du fichier
  //Information : construction de la texture  partir du nom du fichier
  private static final Map<String, STexture> texture_map = new HashMap<String, STexture>();
  
  private String file_name;                 //nom du fichier du modle
  
  STexture texture;
  
  boolean is_read;
  
  /**
   * Constructeur d'un lecteur de texture.
   * 
   * @param file_name - Le nom du fichier de la texture.
   */
  public STextureReader(String file_name)
  {
    this.file_name = file_name;
    texture = DEFAULT_TEXTURE;
    is_read = false;
    
    initialize();
  }

  /**
   * Mthode pour obtenir le nom du fichier en lecture.
   * 
   * @return Le nom du fichier en lecture.
   */
  public String getFileName()
  {
    return file_name;
  }
  
  @Override
  public STexture getValue() throws SRuntimeException
  {
    if(is_read)
      return texture;
    else
      throw new SRuntimeException("Erreur STextureReader 001 : La texture '" + file_name + "' n'a pas t lu.");
  }
  
  @Override
  public boolean asRead()
  {
    return is_read;
  }
  
  /**
   * Mthode permettant de vider la carte de correspondance des textures pralablement charges en mmoire.
   */
  public static void emptyMap()
  {
    texture_map.clear();
  }
  
  /**
   * Mthode pour obtenir une texture lu par le lecteur  partir d'un nom de fichier.
   * 
   * @param file_name Le nom du fichier.
   * @return La texture.
   * @throws IllegalArgumentException Si le nom du fichier n'a pas t lu par le lecteur.
   */
  public static STexture getTexture(String file_name) throws IllegalArgumentException
  {
    if(!texture_map.containsKey(file_name))
      throw new IllegalArgumentException("Erreur STextureReader 002 : La texture '" + file_name + "' n'a pas t lu par le lecteur de texture.");
 
    return texture_map.get(file_name);
  }
 
  /**
   * Mthode pour obtenir l'ensemble des noms de fichier de texture lu par le lecteur de texture.
   * 
   * @return L'ensemble des noms de fichier.
   */
  public static Set<String> getFilesName()
  {
    return texture_map.keySet();
  }
  /**
   * Mthode pour faire l'initialisation du lecteur de texture.
   */
  private void initialize()
  {
    try{
      
      //Vrifier si la texture a dj t lue
      if(texture_map.containsKey(file_name))
        texture = texture_map.get(file_name);
      else
      {
        STextureLoader texture_loader = new STextureLoader();
      
        SLog.logWriteLine("Message STextureReader : Lecture de la texture '" + file_name + "'.");
        
        texture = texture_loader.loadTexture(file_name);  //lecture de la texture (exception lance s'il y a eu erreur)
        
        texture_map.put(file_name, texture);            //mettre le modle lu dans la carte des modles
      }
      
      //Construire une nouvelle texture  partir du mme tableau des couleurs,
      //car il est possible que cette texture ncessite une autre interprtations
      //des coordonnes uv lors de son usage dans un matriel
      texture = new STexture(texture);
      
      is_read = true;
      
    }catch(SLoaderException e){
      SLog.logWriteLine("Erreur STextureReader 002 : Le chargement de la texture '" + file_name + "' est impossible. " + SStringUtil.END_LINE_CARACTER + "\t" + e.getMessage());
    }
  }
  
}//fin de la classe STextureReader
